@@ -12,10 +12,10 @@ These three classes can be directly imported from the toolkit:
     >>> dllnode = ARgorithmToolkit.DoublyLinkedListNode(algo,7)
     >>> dll = ARgorithmToolkit.DoublyLinkedList("dllnode",algo)
     >>> dl = ARgorithmToolkit.List("dl",algo)
-
 """
 
-from ARgorithmToolkit.utils import State, StateSet, ARgorithmError
+from ARgorithmToolkit.utils import ARgorithmHashable, ARgorithmStructure, State, StateSet, ARgorithmError
+from ARgorithmToolkit.encoders import serialize
 
 class DoublyLinkedListNodeState:
     """This class is used to generate states for various actions performed on
@@ -23,11 +23,13 @@ class DoublyLinkedListNodeState:
 
     Attributes:
 
-        name (str) : Name of the variable for whom we are generating states
+        name (str) : Name of the object for which states are generated
+        _id (str) : id of the object for which states are generated
     """
 
-    def __init__(self,name:str):
+    def __init__(self,name:str,_id:str):
         self.name = name
+        self._id = _id
 
     def dllnode_declare(self,value,next_node,prev_node,comments=""):
         """Generates the `dllnode_declare` state when a new node is created.
@@ -43,6 +45,7 @@ class DoublyLinkedListNodeState:
         """
         state_type = "dllnode_declare"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "value" : value,
             "next" : next_node.name if next_node else "none",
@@ -54,7 +57,7 @@ class DoublyLinkedListNodeState:
             comments=comments
         )
 
-    def dllnode_iter(self,value,next_node,prev_node,comments=""):
+    def dllnode_iter(self,value,next_node,prev_node,last_value=None,comments=""):
         """Generates the `dllnode_iter` state when a node is accessed or its
         value is changed.
 
@@ -62,6 +65,7 @@ class DoublyLinkedListNodeState:
             value : The value stored in the linked list node
             next_node (DoublyLinkedListNode): The next pointer
             prev_node (DoublyLinkedListNode): The prev pointer
+            last_value (optional): stores the value in the linked list node before it was changed.
             comments (str, optional): Comments for descriptive purpose. Defaults to "".
 
         Returns:
@@ -69,18 +73,21 @@ class DoublyLinkedListNodeState:
         """
         state_type = "dllnode_iter"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "value" : value,
             "next" : next_node.name if next_node else "none",
             "prev" : prev_node.name if prev_node else "none",
         }
+        if last_value is not None:
+            state_def["last_value"] = last_value
         return State(
             state_type=state_type,
             state_def=state_def,
             comments=comments
         )
 
-    def dllnode_next(self,value,next_node,prev_node,comments=""):
+    def dllnode_next(self,value,next_node,prev_node,last_next,comments=""):
         """Generates the `dllnode_next` state when the next pointer changes.
 
         Args:
@@ -94,10 +101,12 @@ class DoublyLinkedListNodeState:
         """
         state_type = "dllnode_next"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "value" : value,
             "next" : next_node.name if next_node else "none",
             "prev" : prev_node.name if prev_node else "none",
+            "last_next" : last_next.name if last_next else "none",
         }
         return State(
             state_type=state_type,
@@ -105,7 +114,7 @@ class DoublyLinkedListNodeState:
             comments=comments
         )
 
-    def dllnode_prev(self,value,next_node,prev_node,comments=""):
+    def dllnode_prev(self,value,next_node,prev_node,last_prev,comments=""):
         """Generates the `dllnode_prev` state when the prev pointer changes.
 
         Args:
@@ -119,10 +128,12 @@ class DoublyLinkedListNodeState:
         """
         state_type = "dllnode_prev"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "value" : value,
             "next" : next_node.name if next_node else "none",
             "prev" : prev_node.name if prev_node else "none",
+            "last_prev" : last_prev.name if last_prev else "none",
         }
         return State(
             state_type=state_type,
@@ -142,6 +153,7 @@ class DoublyLinkedListNodeState:
         """
         state_type = "dllnode_delete"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
         }
         return State(
@@ -150,7 +162,8 @@ class DoublyLinkedListNodeState:
             comments=comments
         )
 
-class DoublyLinkedListNode:
+@serialize
+class DoublyLinkedListNode(ARgorithmStructure, ARgorithmHashable):
     """The DoublyLinkedListNode class is an implementation of a Linked list
     Node for which we store states. Unlike other data structure classes, in
     which we have to give a name to the instance, we dont have to provide name
@@ -172,18 +185,18 @@ class DoublyLinkedListNode:
         >>> temp = = ARgorithmToolkit.DoublyLinkedListNode(algo,6)
         >>> temp.next = dllnode
         >>> dllnode.prev = temp
-
     """
 
     def __init__(self,algo:StateSet,value=None,comments=""):
-        self.name = id(self)
+        self.name = str(id(self))
+        self._id = str(id(self))
         try:
             assert isinstance(algo,StateSet)
             self.algo = algo
         except AssertionError as e:
             raise ARgorithmError("algo should be of type StateSet") from e
 
-        self.state_generator = DoublyLinkedListNodeState(self.name)
+        self.state_generator = DoublyLinkedListNodeState(self.name, self._id)
 
         self._flag = False
         self.value = value
@@ -205,31 +218,43 @@ class DoublyLinkedListNode:
         """
         if key in ['next','prev'] and value:
             assert isinstance(value,DoublyLinkedListNode) , ARgorithmError("next should be of type None or DoublyLinkedListNode")
+        last_value = None
+        last_prev = None
+        last_next = None
+        if key == 'value' and self._flag:
+            last_value = self.value
+        elif key == 'prev' and self._flag:
+            last_prev = self.prev
+        elif key == 'next' and self._flag:
+            last_next = self.next
         self.__dict__[key] = value
         if key == 'prev' and self._flag:
-            if value or self.prev:
+            if last_prev or self.prev:
                 state = self.state_generator.dllnode_prev(
-                    self.value,
-                    self.next,
-                    self.prev,
-                    "prev pointer updated"
+                    value=self.value,
+                    next_node=self.next,
+                    prev_node=self.prev,
+                    last_prev=last_prev,
+                    comments="prev pointer updated"
                 )
                 self.algo.add_state(state)
         elif key == 'next' and self._flag:
-            if value or self.next:
+            if last_next or self.next:
                 state = self.state_generator.dllnode_next(
-                    self.value,
-                    self.next,
-                    self.prev,
-                    "next pointer updated"
+                    value=self.value,
+                    next_node=self.next,
+                    prev_node=self.prev,
+                    last_next=last_next,
+                    comments="next pointer updated"
                 )
                 self.algo.add_state(state)
         elif key == 'value' and self._flag:
             state = self.state_generator.dllnode_iter(
-                self.value,
-                self.next,
-                self.prev,
-                "value updated"
+                value=self.value,
+                next_node=self.next,
+                prev_node=self.prev,
+                last_value=last_value,
+                comments="value updated"
             )
             self.algo.add_state(state)
 
@@ -254,9 +279,11 @@ class DoublyLinkedListState:
     Attributes:
 
         name (str) : Name of the variable for whom we are generating states
+        _id (str) : id of the variable for whom we are generating states
     """
-    def __init__(self,name:str):
+    def __init__(self,name:str,_id:str):
         self.name = name
+        self._id = _id
 
     def dll_declare(self,head,tail,comments=""):
         """Generates the `dll_declare` state when a new linked list is created.
@@ -271,6 +298,7 @@ class DoublyLinkedListState:
         """
         state_type = "dll_declare"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "head" : head.name if head else "none",
             "tail" : tail.name if tail else "none"
@@ -281,7 +309,7 @@ class DoublyLinkedListState:
             comments=comments
         )
 
-    def dll_head(self,head,tail,comments=""):
+    def dll_head(self,head,tail,last_head=None,comments=""):
         """Generates the `dll_head` state when linked list head is changed.
 
         Args:
@@ -294,17 +322,20 @@ class DoublyLinkedListState:
         """
         state_type = "dll_head"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "head" : head.name if head else "none",
             "tail" : tail.name if tail else "none"
         }
+        if not (last_head is None):
+            state_def['last_head'] = last_head
         return State(
             state_type=state_type,
             state_def=state_def,
             comments=comments
         )
 
-    def dll_tail(self,head,tail,comments=""):
+    def dll_tail(self,head,tail,last_tail=None,comments=""):
         """Generates the `dll_tail` state when linked list tail is changed.
 
         Args:
@@ -317,17 +348,21 @@ class DoublyLinkedListState:
         """
         state_type = "dll_tail"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "head" : head.name if head else "none",
             "tail" : tail.name if tail else "none"
         }
+        if not(last_tail is None):
+            state_def['last_tail'] = last_tail
         return State(
             state_type=state_type,
             state_def=state_def,
             comments=comments
         )
 
-class DoublyLinkedList:
+@serialize
+class DoublyLinkedList(ARgorithmStructure, ARgorithmHashable):
     """The DoublyLinkedList class is used to just store the head of the linked
     list.
 
@@ -347,19 +382,19 @@ class DoublyLinkedList:
         >>> dll = ARgorithmToolkit.DoublyLinkedList("llnode",algo)
         >>> dllnode = ARgorithmToolkit.DoublyLinkedListNode(algo,7)
         >>> dll.head = llnode
-
     """
 
     def __init__(self,name:str,algo:StateSet,head=None,comments=""):
 
         assert isinstance(name,str) , ARgorithmError("Name should be of type string")
         self.name = name
+        self._id = str(id(self))
         try:
             assert isinstance(algo,StateSet)
             self.algo = algo
         except AssertionError as e:
             raise ARgorithmError("algo should be of type StateSet") from e
-        self.state_generator = DoublyLinkedListState(self.name)
+        self.state_generator = DoublyLinkedListState(self.name, self._id)
 
         if head:
             assert self.algo == head.algo, ARgorithmError("The head node belongs to a different StateSet")
@@ -381,12 +416,18 @@ class DoublyLinkedList:
         """
         if key in ['head','tail'] and value:
             assert isinstance(value,DoublyLinkedListNode) , ARgorithmError("next should be of type None or DoublyLinkedListNode")
+        last_head = None
+        last_tail = None
+        if key == 'head' and self._flag:
+            last_head = self.head._id if self.head else "none"
+        elif key == 'tail' and self._flag:
+            last_tail = self.tail._id if self.tail else "none"
         self.__dict__[key] = value
         if key == 'head' and self._flag:
-            state = self.state_generator.dll_head(self.head,self.tail,"head pointer shifts")
+            state = self.state_generator.dll_head(self.head,self.tail,last_head=last_head,comments="head pointer shifts")
             self.algo.add_state(state)
         if key == 'tail' and self._flag:
-            state = self.state_generator.dll_tail(self.head,self.tail,"tail pointer shifts")
+            state = self.state_generator.dll_tail(self.head,self.tail,last_tail=last_tail,comments="tail pointer shifts")
             self.algo.add_state(state)
 
     def __str__(self):
@@ -439,7 +480,6 @@ class List(DoublyLinkedList):
         >>> lis = ARgorithmToolkit.List("list",algo)
         >>> lis
         List([])
-
     """
 
     def __init__(self,name:str,algo:StateSet,comments=""):
@@ -455,7 +495,6 @@ class List(DoublyLinkedList):
         Example:
 
             >>> len(lis)
-
         """
         return self.size
 
@@ -475,7 +514,6 @@ class List(DoublyLinkedList):
             >>> lis.insert(2,1)
             >>> lis
             List([1, 2, 3])
-
         """
         if self.size == 0 or index == 0:
             self.push_front(value)
@@ -535,7 +573,6 @@ class List(DoublyLinkedList):
             >>> lis.push_back(3)
             >>> lis
             List([1, 3])
-
         """
         curr = DoublyLinkedListNode(self.algo,value)
         if self.tail:
@@ -566,7 +603,6 @@ class List(DoublyLinkedList):
             5
             >>> lis
             List([1, 2, 3, 7])
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -595,7 +631,6 @@ class List(DoublyLinkedList):
             7
             >>> lis
             List([1, 2, 3])
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -625,7 +660,6 @@ class List(DoublyLinkedList):
             1
             >>> lis
             List([1, 2, 3])
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -648,7 +682,6 @@ class List(DoublyLinkedList):
             3
             >>> lis
             List([2, 3, 5, 4, 4, 3, 3])
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -664,7 +697,6 @@ class List(DoublyLinkedList):
 
             >>> [x for x in lis]
             [2, 3, 5, 4, 4, 3, 3]
-
         """
         return ListIterator(self)
 
@@ -684,7 +716,6 @@ class List(DoublyLinkedList):
             >>> lis.remove(3)
             >>> lis
             List([2, 5, 4, 4])
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -720,7 +751,6 @@ class List(DoublyLinkedList):
             List([2, 5, 4, 4])
             >>> lis.tolist()
             [2, 5, 4, 4]
-
         """
         curr = self.head
         data = []
