@@ -12,10 +12,10 @@ These three classes can be directly imported from the toolkit:
     >>> llnode = ARgorithmToolkit.LinkedListNode(algo,7)
     >>> ll = ARgorithmToolkit.LinkedList("llnode",algo,llnode)
     >>> fl = ARgorithmToolkit.ForwardList("fl",algo)
-
 """
 
-from ARgorithmToolkit.utils import State, StateSet, ARgorithmError
+from ARgorithmToolkit.utils import ARgorithmHashable, ARgorithmStructure, State, StateSet, ARgorithmError
+from ARgorithmToolkit.encoders import serialize
 
 class LinkedListNodeState:
     """This class is used to generate states for various actions performed on
@@ -24,10 +24,12 @@ class LinkedListNodeState:
     Attributes:
 
         name (str) : Name of the variable for whom we are generating states
+        _id (str) : id of the variable for whom we are generating states
     """
 
-    def __init__(self,name:str):
+    def __init__(self,name:str,_id:str):
         self.name = name
+        self._id = _id
 
     def llnode_declare(self,value,_next,comments=""):
         """Generates the `llnode_declare` state when a new node is created.
@@ -42,6 +44,7 @@ class LinkedListNodeState:
         """
         state_type = "llnode_declare"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "value" : value,
             "next" : _next.name if _next else "none"
@@ -66,6 +69,7 @@ class LinkedListNodeState:
         """
         state_type = "llnode_iter"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "value" : value,
             "next" : _next.name if _next else "none"
@@ -91,6 +95,7 @@ class LinkedListNodeState:
         """
         state_type = "llnode_next"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "value" : value,
             "next" : _next.name if _next else "none",
@@ -113,6 +118,7 @@ class LinkedListNodeState:
         """
         state_type = "llnode_delete"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
         }
         return State(
@@ -121,7 +127,8 @@ class LinkedListNodeState:
             comments=comments
         )
 
-class LinkedListNode:
+@serialize
+class LinkedListNode(ARgorithmStructure, ARgorithmHashable):
     """The LinkedListNode class is an implementation of a Linked list Node for
     which we store states. Unlike other data structure classes, in which we
     have to give a name to the instance, we dont have to provide name in the
@@ -142,18 +149,18 @@ class LinkedListNode:
         >>> temp = = ARgorithmToolkit.LinkedListNode(algo,7)
         >>> temp.next = llnode
         >>> llnode = temp
-
     """
 
     def __init__(self,algo:StateSet,value=None,comments=""):
-        self.name = id(self)
+        self.name = str(id(self))
+        self._id = str(id(self))
         try:
             assert isinstance(algo,StateSet)
             self.algo = algo
         except AssertionError as e:
             raise ARgorithmError("algo should be of type StateSet") from e
 
-        self.state_generator = LinkedListNodeState(self.name)
+        self.state_generator = LinkedListNodeState(self.name, self._id)
 
         self._flag = False
         self.value = value
@@ -220,9 +227,11 @@ class LinkedListState:
     Attributes:
 
         name (str) : Name of the variable for whom we are generating states
+        _id (str) : id of the variable for whom we are generating states
     """
-    def __init__(self,name:str):
+    def __init__(self,name:str,_id:str):
         self.name = name
+        self._id = _id
 
     def ll_declare(self,head,comments=""):
         """Generates the `ll_declare` state when a new linkedlist is created.
@@ -236,6 +245,7 @@ class LinkedListState:
         """
         state_type = "ll_declare"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "head" : head.name if head else "none"
         }
@@ -245,7 +255,7 @@ class LinkedListState:
             comments=comments
         )
 
-    def ll_head(self,head,comments=""):
+    def ll_head(self,head,last_head=None,comments=""):
         """Generates the `ll_head` state when linkedlist head is changed.
 
         Args:
@@ -257,16 +267,20 @@ class LinkedListState:
         """
         state_type = "ll_head"
         state_def = {
+            "id" : self._id,
             "variable_name" : self.name,
             "head" : head.name if head else "none"
         }
+        if not (last_head is None):
+            state_def["last_head"] = last_head
         return State(
             state_type=state_type,
             state_def=state_def,
             comments=comments
         )
 
-class LinkedList:
+@serialize
+class LinkedList(ARgorithmStructure, ARgorithmHashable):
     """The LinkedList class is used to just store the head of the linked list.
 
     This class is useful when programmer want to program his own List class using
@@ -285,19 +299,19 @@ class LinkedList:
         >>> ll = ARgorithmToolkit.LinkedList("llnode",algo)
         >>> llnode = ARgorithmToolkit.LinkedListNode(algo,7)
         >>> ll.head = llnode
-
     """
 
     def __init__(self,name:str,algo:StateSet,head=None,comments=""):
 
         assert isinstance(name,str) , ARgorithmError("Name should be of type string")
         self.name = name
+        self._id = str(id(self))
         try:
             assert isinstance(algo,StateSet)
             self.algo = algo
         except AssertionError as e:
             raise ARgorithmError("algo should be of type StateSet") from e
-        self.state_generator = LinkedListState(self.name)
+        self.state_generator = LinkedListState(self.name, self._id)
 
         self._flag = False
         if head:
@@ -315,11 +329,14 @@ class LinkedList:
         Raises:
             ARgorithmError: Raised if head pointer is not type None or LinkedListNode
         """
+        last_head = None
         if key == 'head' and value:
             assert isinstance(value,LinkedListNode) , ARgorithmError("next should be of type None or LinkedListNode")
+        if key == 'head' and self._flag:
+            last_head = self.head._id if self.head else "none"
         self.__dict__[key] = value
         if key == 'head' and self._flag:
-            state = self.state_generator.ll_head(self.head,"head pointer shifts")
+            state = self.state_generator.ll_head(self.head,last_head=last_head, comments="head pointer shifts")
             self.algo.add_state(state)
 
     def __str__(self):
@@ -368,7 +385,6 @@ class ForwardList(LinkedList):
     Example:
 
         >>> fl = ARgorithmToolkit.ForwardList("fl",algo)
-
     """
 
     def __init__(self,name:str,algo:StateSet,comments=""):
@@ -387,7 +403,6 @@ class ForwardList(LinkedList):
             >>> fl.push_front(1)
             >>> fl.size()
             1
-
         """
         return self.size
 
@@ -407,7 +422,6 @@ class ForwardList(LinkedList):
             >>> fl.insert(3,1)
             >>> fl
             ForwardList([4, 3, 2])
-
         """
         if self.size == 0 or index == 0:
             self.push_front(value)
@@ -442,7 +456,6 @@ class ForwardList(LinkedList):
             >>> fl.push_front(1)
             >>> fl
             ForwardList([1])
-
         """
         curr = LinkedListNode(self.algo,value)
         if self.head:
@@ -468,7 +481,6 @@ class ForwardList(LinkedList):
             ForwardList([2, 1])
             >>> fl.pop_front()
             2
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -494,7 +506,6 @@ class ForwardList(LinkedList):
             ForwardList([2, 1])
             >>> fl.front()
             2
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -511,7 +522,6 @@ class ForwardList(LinkedList):
 
             >>> [x for x in fl]
             [2, 2, 1]
-
         """
         return ForwardListIterator(self)
 
@@ -531,7 +541,6 @@ class ForwardList(LinkedList):
             >>> fl.remove(2)
             >>> fl
             ForwardList([1])
-
         """
         if self.head is None:
             raise ARgorithmError("Empty list")
@@ -565,7 +574,6 @@ class ForwardList(LinkedList):
             ForwardList([3, 1])
             >>> fl.tolist()
             [3, 1]
-
         """
         curr = self.head
         data = []
